package sensorComponent;

import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;
import gamf.framework.exceptions.EventGenerationException;
import gamf.framework.exceptions.TypeMissMatch;
import gamf.interfaces.systemAdapter.IEffector;
import gamf.interfaces.systemAdapter.IEventGenerator;

/**
 * A simplified target system which periodically simulates the probing for
 * temperature measurements (from deployed sensor nodes). During the processing
 * of the data, errors are simulated. It is assumed that depending on the usage
 * rate of the sensor nodes by other applications the monitored data is faulty.
 * (i.e. the higher the probing interval the small the possibility that an error
 * is simulated). In such a case an increase of the polling interval would be
 * beneficial. In any other situation a probing interval of ~100 ms is assumed
 * to be ideal.
 * 
 * An event generator generates (GAMF) events when errors are monitored. The
 * target system implements an effector via which the probing interval can be
 * adapted.
 * 
 * 
 * @author Markus
 * 
 */

public class MyTargetSystem extends Thread implements IEffector {

	IEventGenerator eg;
	int probingInterval = Constants.initialProbingInterval;
	private boolean running = true;

	class MonitoringData {
		private String descr;
		private int value;

		public MonitoringData(String descr, int value) {
			this.descr = descr;
			this.value = value;
		}

		public String getDescr() {
			return descr;
		}

		public int getValue() {
			return value;
		}

		@Override
		public String toString() {
			return descr + ":" + value;
		}

	}

	/**
	 * unmanaged target system
	 */
	public MyTargetSystem() {

	}

	/**
	 * managed target system
	 */
	public MyTargetSystem(IEventGenerator eg) {
		this.eg = eg;

	}

	@Override
	public void run() {
		while (running) {
			super.run();
			MonitoringData md = gatherMonitoringData();
			try {
				processData(md);
			} catch (Exception e) {
				try {
					if(eg!=null){
					eg.triggerEventGeneration(Constants.ERROR_EVENT);
					}else{
						Diagnostic.trace("target system operates unmanaged");
					}
				} catch (EventGenerationException e1) {
					e1.printStackTrace();
				}
				e.printStackTrace();
			}

			try {
				sleep(probingInterval);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void processData(MonitoringData md) throws Exception {
		// a high interval causes less errors:
		// ////////////////////////////////////

		// -> interval is normalised between 0 -1
		float k = 500;
		float normailzed_interval = (1 / ((float) probingInterval / k + 1));

		// if temperature is in a specific range we generate errors
		int upperRange = Math.round(normailzed_interval * 100);
		if (md.getValue() > upperRange) {
			Diagnostic.trace("simulating error");
			throw new Exception("simulating error");
		}

		// the "size" of the range is determined by the normalised probing
		// interval.
		// the higher the interval the less errors, i.e.
		// i.e. the higher the interval the smaller the range which causes an
		// error

		Diagnostic.trace(DiagnosticLevel.FULL, "processing " + md
				+ "(uper range:" + upperRange + ")");

	}

	private MonitoringData gatherMonitoringData() {

		double randomValue = Math.random();
		int randomTemp = (int) (100 * randomValue);
		return new MonitoringData("Temperature", randomTemp);
	}

	public Object get() {
		return probingInterval;
	}

	public void set(Object value) throws TypeMissMatch {
		if (value instanceof Integer) {
			probingInterval = (Integer) value;
		} else {
			throw new TypeMissMatch(
					"the probing interval value must be of type Integer");
		}
	}

	public String getSystemAdapterCategory() {
		return Constants.SYSTEM_ADAPTER_CATEGORY;
	}

	public String getSystemAdapterIdentifier() {
		return Constants.SENSOR_COMPONENT;
	}

	public static void main(String[] args) {
		MyTargetSystem mts = new MyTargetSystem();

		mts.start();
	}

}
